#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.widget.principal module

This module defines widgets to select principals.
"""

from zope.interface import implementer_only
from zope.schema.vocabulary import SimpleTerm

from pyams_form.browser.select import SelectWidget
from pyams_form.converter import SequenceDataConverter
from pyams_form.interfaces import IDataConverter
from pyams_form.interfaces.widget import IFieldWidget
from pyams_form.widget import FieldWidget
from pyams_layer.interfaces import IFormLayer
from pyams_security.interfaces import ISecurityManager
from pyams_security.schema import IPrincipalField, IPrincipalsSetField
from pyams_security_views.widget.interfaces import IPrincipalWidget, IPrincipalsSetWidget
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_utility


__docformat__ = 'restructuredtext'

from pyams_security_views import _


def principal_term_factory(value):
    """Principal term factory"""
    manager = get_utility(ISecurityManager)
    principal = manager.get_principal(value)
    if principal is not None:
        return SimpleTerm(principal.id, title=principal.title)
    return None


#
# Principal widget
#

@adapter_config(required=(IPrincipalField, IPrincipalWidget),
                provides=IDataConverter)
class PrincipalDataConverter(SequenceDataConverter):
    """Principal data converter"""

    def to_widget_value(self, value):
        """Value to widget converter"""
        if isinstance(value, set):
            try:
                value = next(iter(value))
            except StopIteration:
                value = None
        return super().to_widget_value(value)


@implementer_only(IPrincipalWidget)
class PrincipalWidget(SelectWidget):
    """Principal widget"""

    placeholder = _("No selected principal")
    separator = '|'

    ajax_url = '/api/security/principals'

    @staticmethod
    def term_factory(value):
        """Selected principals terms factory"""
        return principal_term_factory(value)


@adapter_config(required=(IPrincipalField, IFormLayer),
                provides=IFieldWidget)
def PrincipalFieldWidget(field, request):  # pylint: disable=invalid-name
    """Principal field widget factory"""
    return FieldWidget(field, PrincipalWidget(request))


#
# Principals set widget
#

@implementer_only(IPrincipalsSetWidget)
class PrincipalsSetWidget(PrincipalWidget):
    """Principals set widget"""

    multiple = 'multiple'


@adapter_config(required=(IPrincipalsSetField, IFormLayer),
                provides=IFieldWidget)
def PrincipalsSetFieldWidget(field, request):  # pylint: disable=invalid-name
    """Principals set field widget factory"""
    return FieldWidget(field, PrincipalsSetWidget(request))



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.widget.permission module

This module defines widgets to select permissions.
"""

from zope.interface import implementer_only

from pyams_form.browser.select import SelectWidget
from pyams_form.converter import CollectionSequenceDataConverter
from pyams_form.interfaces import IDataConverter
from pyams_form.interfaces.widget import IFieldWidget
from pyams_form.widget import FieldWidget
from pyams_layer.interfaces import IFormLayer
from pyams_security.schema import IPermissionField, IPermissionsSetField
from pyams_security_views.widget.interfaces import IPermissionWidget, IPermissionsSetWidget
from pyams_utils.adapter import adapter_config


__docformat__ = 'restructuredtext'


#
# Permission widget
#

@implementer_only(IPermissionWidget)
class PermissionWidget(SelectWidget):
    """Principal widget"""


@adapter_config(required=(IPermissionField, IFormLayer),
                provides=IFieldWidget)
def PermissionFieldWidget(field, request):  # pylint: disable=invalid-name
    """Permission field widget factory"""
    return FieldWidget(field, PermissionWidget(request))


#
# Permissions set widget
#

@adapter_config(required=(IPermissionsSetField, IPermissionsSetWidget),
                provides=IDataConverter)
class PermissionsSetDataConverter(CollectionSequenceDataConverter):
    """Permissions set data converter"""


@implementer_only(IPermissionsSetWidget)
class PermissionsSetWidget(SelectWidget):
    """Permissions set widget"""

    multiple = True


@adapter_config(required=(IPermissionsSetField, IFormLayer),
                provides=IFieldWidget)
def PermissionsSetFieldWidget(field, request):  # pylint: disable=invalid-name
    """Permissions set field widget factory"""
    return FieldWidget(field, PermissionsSetWidget(request))



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.widget.interfaces module

This module defines public interfaces of permissions and principals selection widgets.
"""

from pyams_form.interfaces.widget import ISelectWidget
from pyams_skin.interfaces.widget import IDynamicSelectWidget


__docformat__ = 'restructuredtext'


class IPermissionWidget(ISelectWidget):
    """Permission widget"""


class IPermissionsSetWidget(ISelectWidget):
    """Permissions set widget"""


class IPrincipalWidget(IDynamicSelectWidget):
    """Principal widget"""


class IPrincipalsSetWidget(IDynamicSelectWidget):
    """Principals set widget"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.widget module

This modules defines widgets which are used to display security plug-ins.
"""

__docformat__ = 'restructuredtext'

from pyramid.decorator import reify
from zope.interface import implementer_only
from zope.schema.interfaces import ITuple
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_form.interfaces import ITerms
from pyams_form.term import Terms
from pyams_form.widget import FieldWidget
from pyams_security.interfaces import IPlugin, ISecurityManager
from pyams_security_views.zmi.interfaces import ISecurityManagerPluginsWidget, \
    ISecurityPropertiesEditForm
from pyams_skin.widget.list import OrderedListWidget
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import query_utility
from pyams_zmi.interfaces import IAdminLayer


@implementer_only(ISecurityManagerPluginsWidget)
class SecurityManagerPluginsWidget(OrderedListWidget):
    """Security manager plugins widget"""


def SecurityManagerPluginsFieldWidget(field, request):  # pylint: disable=invalid-name
    """Security manager plugins field widget"""
    return FieldWidget(field, SecurityManagerPluginsWidget(request))


@adapter_config(required=(ISecurityManager, IAdminLayer, ISecurityPropertiesEditForm,
                          ITuple, ISecurityManagerPluginsWidget),
                provides=ITerms)
class SecurityManagerPluginsTerms(Terms):
    """Security manager plugins terms"""

    def __init__(self, context, request, form, field, widget):  # pylint: disable=too-many-arguments
        self.context = context
        self.request = request
        self.form = form
        self.field = field
        self.widget = widget

    @reify
    def terms(self):
        """Inner terms getter"""
        field = self.field.bind(self.context)
        terms = []
        for name in field.get(self.context):
            plugin = query_utility(IPlugin, name=name)
            if plugin is None:
                plugin = self.context.get(name)
            if plugin is not None:
                terms.append(SimpleTerm(name,
                                        title=self.request.localizer.translate(plugin.title)))
        return SimpleVocabulary(terms)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.login module

This module provides views and content providers used to manage login page configuration.
"""

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION
from pyams_security_views.interfaces.login import ILoginConfiguration
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.helper.event import get_json_widget_refresh_callback
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.zmi.interfaces import IConfigurationMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


@viewletmanager_config(name='login-form-configuration.menu', context=ISiteRoot, layer=IAdminLayer,
                       manager=IConfigurationMenu, weight=20,
                       permission=MANAGE_SYSTEM_PERMISSION)
class LoginFormConfigurationMenu(NavigationMenuItem):
    """Login form configuration menu"""

    label = _("Login form")
    href = '#login-form-configuration.html'


@ajax_form_config(name='login-form-configuration.html', context=ISiteRoot, layer=IPyAMSLayer,
                  permission=MANAGE_SYSTEM_PERMISSION)
class LoginFormConfigurationForm(AdminEditForm):
    """Login form configuration form"""

    title = _("Login form configuration")
    legend = _("Form properties")

    fields = Fields(ILoginConfiguration)

    def get_content(self):
        return ILoginConfiguration(self.context)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if 'header' in self.widgets:
            self.widgets['header'].set_widgets_attr('rows', 5)
            self.widgets['header'].add_widgets_class('monospace')
        if 'footer' in self.widgets:
            self.widgets['footer'].set_widgets_attr('rows', 5)
            self.widgets['footer'].add_widgets_class('monospace')


@adapter_config(required=(ISiteRoot, IAdminLayer, LoginFormConfigurationForm),
                provides=IAJAXFormRenderer)
class LoginFormConfigurationRenderer(ContextRequestViewAdapter):
    """Login form configuration form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes and ('logo' in changes.get(ILoginConfiguration, ())):
            return get_json_widget_refresh_callback(self.view, 'logo', self.request)
        return None



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.policy module

This module defines views and content providers which are used to manage roles and
security policy
"""

from zope.component import getAdapters
from zope.interface import Interface

from pyams_form.ajax import ajax_form_config
from pyams_form.browser.checkbox import SingleCheckBoxFieldWidget
from pyams_form.field import Fields
from pyams_form.interfaces import DISPLAY_MODE, INPUT_MODE
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import IDefaultProtectionPolicy, IProtectedObject, IRolesPolicy
from pyams_security.interfaces.base import IRole, MANAGE_SECURITY_PERMISSION, \
    VIEW_SYSTEM_PERMISSION
from pyams_security_views.zmi.interfaces import IObjectSecurityMenu
from pyams_skin.interfaces.viewlet import IHelpViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_utils.registry import get_utility
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import ISiteManagementMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


#
# Object roles views
#

@viewletmanager_config(name='object-roles.menu', context=IDefaultProtectionPolicy,
                       layer=IAdminLayer, manager=ISiteManagementMenu, weight=900,
                       permission=VIEW_SYSTEM_PERMISSION,
                       provides=IObjectSecurityMenu)
class ObjectSecurityMenu(NavigationMenuItem):
    """Object security menu"""

    label = _("User roles")
    icon_class = 'fas fa-users'
    href = '#object-roles.html'


@ajax_form_config(name='object-roles.html', context=IDefaultProtectionPolicy,
                  layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class ProtectedObjectRolesEditForm(AdminEditForm):
    """Protected object roles edit form"""

    title = _("User roles")
    legend = _("Granted object roles")

    @property
    def fields(self):
        """Form fields getter"""
        fields = Fields(Interface)
        for _name, policy in sorted(getAdapters((self.context,), IRolesPolicy),
                                    key=lambda x: x[1].weight):
            fields += Fields(policy.roles_interface)
        return fields

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if self.mode == DISPLAY_MODE:
            return
        principals = self.request.effective_principals
        for widget in self.widgets.values():
            widget.mode = DISPLAY_MODE
            role = get_utility(IRole, name=widget.field.role_id)
            if role.managers:
                for manager in role.managers:
                    if manager in principals:
                        widget.mode = INPUT_MODE
                        break


@viewlet_config(name='object-roles.alert', context=IDefaultProtectionPolicy,
                layer=IAdminLayer, view=ProtectedObjectRolesEditForm,
                manager=IHelpViewletManager, weight=1)
class ProtectedObjectRolesAlert(AlertMessage):
    """Protected object roles alert"""

    def __new__(cls, context, request, view, manager):  # pylint: disable=unused-argument
        if not view.widgets:
            return AlertMessage.__new__(cls)
        return None

    status = 'danger'
    _message = _("Roles are not defined for this context!")


#
# Security policy views
#

@viewlet_config(name='security-policy.menu', context=IDefaultProtectionPolicy,
                layer=IAdminLayer, manager=IObjectSecurityMenu,
                permission=MANAGE_SECURITY_PERMISSION, weight=10)
class ProtectedObjectSecurityPolicyMenuItem(NavigationMenuItem):
    """Protected object security policy menu item"""

    label = _("Security policy")
    icon_class = 'fas fa-key'
    href = '#security-policy.html'


@ajax_form_config(name='security-policy.html', context=IDefaultProtectionPolicy,
                  layer=IPyAMSLayer, permission=MANAGE_SECURITY_PERMISSION)
class ProtectedObjectSecurityPolicyEditForm(AdminEditForm):
    """Protected object security policy edit form"""

    title = _("Security manager")
    legend = _("Security policy management")

    fields = Fields(IProtectedObject)
    fields['inherit_parent_security'].widget_factory = SingleCheckBoxFieldWidget
    fields['inherit_parent_roles'].widget_factory = SingleCheckBoxFieldWidget

    def get_content(self):
        return IProtectedObject(self.context)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.skin.login module

This modules defines login and modal login views.
These views are automatically associated with Pyramid forbidden views.
"""

from pyramid.csrf import new_csrf_token
from pyramid.decorator import reify
from pyramid.events import subscriber
from pyramid.httpexceptions import HTTPForbidden, HTTPFound
from pyramid.response import Response
from pyramid.security import forget, remember
from pyramid.view import forbidden_view_config, view_config
from zope.interface import Interface, Invalid, implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.form import AddForm
from pyams_form.interfaces import HIDDEN_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer, IResources
from pyams_security.credential import Credentials
from pyams_security.interfaces import ISecurityManager, IViewContextPermissionChecker, \
    LOGIN_REFERER_KEY
from pyams_security.interfaces.base import PUBLIC_PERMISSION
from pyams_security_views.interfaces.login import ILoginConfiguration, ILoginFormButtons, \
    ILoginFormFields, ILoginView, IModalLoginFormButtons
from pyams_skin.interfaces.view import IModalFullPage, IModalPage
from pyams_skin.interfaces.viewlet import IFooterViewletManager, IHeaderViewletManager
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.registry import query_utility
from pyams_utils.text import text_to_html
from pyams_viewlet.viewlet import Viewlet, viewlet_config


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


@forbidden_view_config(request_type=IPyAMSLayer)
def ForbiddenView(request):  # pylint: disable=invalid-name
    """Default forbidden view"""
    request.session[LOGIN_REFERER_KEY] = request.url
    return HTTPFound('login.html')


@forbidden_view_config(request_type=IPyAMSLayer, renderer='json', xhr=True)
def ForbiddenAJAXView(request):  # pylint: disable=invalid-name
    """AJAX forbidden view"""
    request.response.status = HTTPForbidden.code
    return {
        'status': 'modal',
        'location': 'login-dialog.html'
    }


@ajax_form_config(name='login.html', layer=IPyAMSLayer)  # pylint: disable=abstract-method
@implementer(IModalFullPage, ILoginView, IObjectData)
class LoginForm(AddForm):
    """Login form"""

    prefix = 'login_form.'
    title = _("You must authenticate")
    legend = _("Please enter valid credentials")

    modal_class = FieldProperty(IModalFullPage['modal_class'])

    fields = Fields(ILoginFormFields)
    buttons = Buttons(ILoginFormButtons)

    edit_permission = None

    object_data = {
        'ams-warn-on-change': False,
        'ams-modules': 'callbacks helpers',
        'ams-callback': 'MyAMS.helpers.setLoginHash'
    }

    def update(self):
        super().update()
        new_csrf_token(self.request)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        hash = self.widgets.get('hash')
        if hash is not None:
            hash.mode = HIDDEN_MODE

    @handler(buttons['login'])
    def login_handler(self, action):  # pylint: disable=unused-argument
        """Login button handler"""
        data, errors = self.extract_data()
        if errors:
            self.status = self.form_errors_message
            return None
        principal_id = data.get('principal_id')
        if principal_id is not None:
            request = self.request
            headers = remember(request, principal_id)
            response = request.response
            response.headerlist.extend(headers)
            if not self.request.is_xhr:
                response.status_code = 302
                session = request.session
                hash = data.get('hash', '')
                if LOGIN_REFERER_KEY in session:
                    response.location = f'{session[LOGIN_REFERER_KEY]}{hash}'
                    del session[LOGIN_REFERER_KEY]
                else:
                    response.location = f'/{hash}'
            return response
        return None


@ajax_form_config(name='login-dialog.html', layer=IPyAMSLayer)  # pylint: disable=abstract-method
@implementer(IModalPage, ILoginView)
class ModalLoginForm(LoginForm):
    """Modal login form"""

    modal_class = 'modal-lg'
    buttons = Buttons(IModalLoginFormButtons)


@subscriber(IDataExtractedEvent, form_selector=ILoginView)
def handle_login_form_data(event):
    """Check credentials after data extraction"""
    data = event.data
    if 'principal_id' in data:
        del data['principal_id']
    sm = query_utility(ISecurityManager)  # pylint: disable=invalid-name
    if sm is None:
        event.form.widgets.errors += (Invalid(_("Missing security manager utility. "
                                                "Please contact your system administrator!")), )
    else:
        credentials = Credentials('form', id=data['login'], **data)
        principal_id = sm.authenticate(credentials, event.form.request)
        if principal_id is None:
            event.form.widgets.errors += (Invalid(_("Invalid credentials!")),)
        else:
            data['principal_id'] = principal_id


@adapter_config(required=(Interface, IPyAMSLayer, ILoginView),
                provides=IAJAXFormRenderer)
class LoginFormAJAXRenderer(ContextRequestViewAdapter):
    """Login form result renderer"""

    def render(self, changes):  # pylint: disable=unused-argument
        """AJAX form renderer"""
        status = {'status': 'redirect'}
        hash = self.request.params.get('login_form.widgets.hash', '')
        session = self.request.session
        if LOGIN_REFERER_KEY in session:
            status['location'] = f"{session[LOGIN_REFERER_KEY] or '/'}{hash}"
            del session[LOGIN_REFERER_KEY]
        else:
            status['location'] = f'/{hash}'
        return status


try:
    from pyams_zmi.interfaces.configuration import IZMIConfiguration, MYAMS_BUNDLES

    @adapter_config(name='login',
                    required=(Interface, IPyAMSLayer, ILoginView),
                    provides=IResources)
    class LoginViewResourcesAdapter(ContextRequestViewAdapter):
        """Login view resources adapter"""

        weight = 10

        @property
        def resources(self):
            """Resources getter"""
            request = self.request
            configuration = IZMIConfiguration(request.root, None)
            if configuration is not None:
                # yield MyAMS bundle
                bundle, _label = MYAMS_BUNDLES.get(configuration.myams_bundle)
                yield bundle

except ImportError:
    pass


@viewlet_config(name='login.logo', layer=IPyAMSLayer, view=ILoginView,
                manager=IHeaderViewletManager, weight=1)
@template_config(template='templates/login-logo.pt')
class LoginLogoViewlet(Viewlet):
    """Login logo viewlet"""

    @property
    def logo(self):
        """Logo getter"""
        configuration = ILoginConfiguration(self.request.root, None)
        if configuration:
            return II18n(configuration).query_attribute('logo', request=self.request)
        return None


@template_config(template='templates/login-viewlet.pt')
class LoginViewlet(Viewlet):
    """Base login viewlet"""

    text_value = None
    attribute_name = 'header'
    renderer_getter = lambda x, y: y

    @reify
    def configuration(self):
        """Configuration getter"""
        return ILoginConfiguration(self.request.root, None)

    def render(self):
        configuration = self.configuration
        if configuration:
            # pylint: disable=assignment-from-no-return
            value = II18n(configuration).query_attribute(self.attribute_name,
                                                         request=self.request)
            if value:
                renderer = self.renderer_getter(configuration)  # pylint: disable=no-value-for-parameter
                if renderer == 'text':
                    self.text_value = value
                    return super().render()
                return text_to_html(value, renderer=renderer)
        return ''


@viewlet_config(name='login.header', layer=IPyAMSLayer, view=ILoginView,
                manager=IHeaderViewletManager, weight=100)
class LoginHeaderViewlet(LoginViewlet):
    """Login header viewlet"""

    attribute_name = 'header'
    renderer_getter = lambda x, config: config.header_renderer


@viewlet_config(name='login.footer', layer=IPyAMSLayer, view=ILoginView,
                manager=IFooterViewletManager, weight=100)
class LoginFooterViewlet(LoginViewlet):
    """Login footer viewlet"""

    attribute_name = 'footer'
    renderer_getter = lambda x, config: config.footer_renderer


@view_config(name='logout', request_type=IPyAMSLayer)
def logout(request):
    """Logout view"""
    headers = forget(request)
    response = Response()
    response.headerlist.extend(headers)
    response.status_code = 302
    response.location = request.referer or '/'
    return response



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.api.principal module

This module only provides a small Cornice API to search for principals.
"""

import sys

from colander import MappingSchema, SchemaNode, SequenceSchema, String
from cornice import Service
from cornice.validators import colander_querystring_validator
from pyramid.httpexceptions import HTTPOk

from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_security.rest import check_cors_origin, set_cors_headers
from pyams_security_views.interfaces import REST_PRINCIPALS_SEARCH_ROUTE
from pyams_utils.registry import query_utility


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


TEST_MODE = sys.argv[-1].endswith('/test')


class PrincipalsSearchQuerySchema(MappingSchema):
    """Principals search schema"""
    term = SchemaNode(String(),
                      description=_("Principals search string"))


class PrincipalResultSchema(MappingSchema):
    """Principal result schema"""
    id = SchemaNode(String(),
                    description=_("Principal ID"))
    text = SchemaNode(String(),
                      description=_("Principal title"))


class PrincipalsSearchResults(SequenceSchema):
    """Principals search results interface"""
    result = PrincipalResultSchema()


class PrincipalsSearchResultsSchema(MappingSchema):
    """Principals search results schema"""
    results = PrincipalsSearchResults(description=_("Results list"))


search_responses = {
    HTTPOk.code: PrincipalsSearchResultsSchema(description=_("Search results")),
}
if TEST_MODE:
    service_params = {}
else:
    service_params = {
        'response_schemas': search_responses
    }


service = Service(name=REST_PRINCIPALS_SEARCH_ROUTE,
                  pyramid_route=REST_PRINCIPALS_SEARCH_ROUTE,
                  description="Principals management")


@service.options(validators=(check_cors_origin, set_cors_headers),
                 **service_params)
def principals_options(request):  # pylint: disable=unused-argument
    """Principals service options"""
    return ''


@service.get(permission=VIEW_SYSTEM_PERMISSION,
             schema=PrincipalsSearchQuerySchema(),
             validators=(check_cors_origin, colander_querystring_validator, set_cors_headers),
             **service_params)
def get_principals(request):
    """Returns list of principals matching given query"""
    if TEST_MODE:
        query = request.params.get('term')
    else:
        query = request.validated.get('term')
    if not query:
        return []
    manager = query_utility(ISecurityManager)
    return {
        'results': [{
            'id': principal.id,
            'text': principal.title
        } for principal in manager.find_principals(query)]
    }



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface
from zope.schema import Choice, Password, TextLine

from pyams_file.schema import ImageField
from pyams_i18n.schema import I18nTextField
from pyams_skin.schema.button import CloseButton, ResetButton, SubmitButton
from pyams_utils.text import PYAMS_HTML_RENDERERS_VOCABULARY


__docformat__ = 'restructuredtext'

from pyams_security_views import _


class ILoginConfiguration(Interface):
    """Login configuration interface"""

    logo = ImageField(title=_("Login logo"),
                      description=_("Image used in login form"),
                      required=False)

    header = I18nTextField(title=_("Login header"),
                           description=_("This text will be displayed in login page header"),
                           required=False)

    header_renderer = Choice(title=_("Header renderer"),
                             description=_("Text renderer used for the header"),
                             required=True,
                             vocabulary=PYAMS_HTML_RENDERERS_VOCABULARY,
                             default='text')

    footer = I18nTextField(title=_("Login footer"),
                           description=_("This text will be displayed in login page footer"),
                           required=False)

    footer_renderer = Choice(title=_("Footer renderer"),
                             description=_("Text renderer used for the footer"),
                             required=True,
                             vocabulary=PYAMS_HTML_RENDERERS_VOCABULARY,
                             default='text')


class ILoginView(Interface):
    """Login view marker interface"""


class ILoginFormFields(Interface):
    """Login form fields"""

    hash = TextLine(title=_("Redirection hash"),
                    required=False)

    login = TextLine(title=_("Login"))

    password = Password(title=_("Password"))


class ILoginFormButtons(Interface):
    """Login form buttons"""

    login = SubmitButton(name='login',
                         title=_("Connect"))

    reset = ResetButton(name='reset',
                        title=_("Reset"))


class IModalLoginFormButtons(Interface):
    """Modal login form buttons"""

    login = SubmitButton(name='login',
                         title=_("Connect"))

    close = CloseButton(name='close',
                        title=_("Cancel"))



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_file.property import FileProperty
from pyams_security_views.interfaces.login import ILoginConfiguration
from pyams_site.interfaces import ISiteEtcTraverser, ISiteRoot
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'


@factory_config(ILoginConfiguration)
class LoginConfiguration(Persistent, Contained):
    """Site login configuration"""

    logo = FileProperty(ILoginConfiguration['logo'])
    header = FieldProperty(ILoginConfiguration['header'])
    header_renderer = FieldProperty(ILoginConfiguration['header_renderer'])
    footer = FieldProperty(ILoginConfiguration['footer'])
    footer_renderer = FieldProperty(ILoginConfiguration['footer_renderer'])


LOGIN_CONFIGURATION_KEY = 'pyams_security.login.configuration'
"""Annotations key used to store login configuration"""


@adapter_config(required=ISiteRoot, provides=ILoginConfiguration)
def site_root_login_configuration_factory(context):
    """Site root login configuration factory"""
    return get_annotation_adapter(context, LOGIN_CONFIGURATION_KEY, ILoginConfiguration,
                                  name='++etc++login.configuration')


@adapter_config(name='login.configuration', required=ISiteRoot, provides=ISiteEtcTraverser)
def site_root_login_configuration_traverser(context):
    """Site root ++etc++login.configuration traverser extension"""
    return ILoginConfiguration(context, None)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi base module

This module provides views and content providers used to display security manager plug-ins.
"""

from pyramid.view import view_config
from zope.interface import Interface, implementer

from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import MANAGE_SECURITY_PERMISSION
from pyams_security.interfaces.names import ADMIN_USER_NAME, INTERNAL_USER_NAME
from pyams_security.interfaces.plugin import IDirectorySearchPlugin
from pyams_security_views.zmi.interfaces import ISecurityManagerView, ISecurityMenu
from pyams_site.interfaces import ISiteRoot
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_utility, query_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.helper.container import delete_container_element
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IControlPanelMenu
from pyams_zmi.table import ActionColumn, IconColumn, NameColumn, Table, TableAdminView, \
    TableElementEditor, TrashColumn
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


@viewlet_config(name='security.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IControlPanelMenu, weight=20,
                permission=MANAGE_SECURITY_PERMISSION)
class SecurityMenu(NavigationMenuItem):
    """Security menu"""

    label = _("Security")
    icon_class = 'fas fa-user-lock'

    def __new__(cls, context, request, view, manager):  # pylint: disable=unused-arguments
        sm = query_utility(ISecurityManager)  # pylint: disable=invalid-name
        if (sm is None) or not sm.show_home_menu:
            return None
        return NavigationMenuItem.__new__(cls)

    def get_href(self):
        """Menu URL getter"""
        sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
        return absolute_url(sm, self.request, 'admin')


@viewletmanager_config(name='security-plugins.menu',
                       context=ISecurityManager, layer=IAdminLayer,
                       manager=IControlPanelMenu, weight=10,
                       permission=MANAGE_SECURITY_PERMISSION,
                       provides=ISecurityMenu)
class SecurityPluginsMenu(NavigationMenuItem):
    """Security plug-ins menu"""

    label = _("Security")
    icon_class = 'fas fa-user-lock'

    href = '#security-plugins.html'


class SecurityPluginsTable(Table):
    """Security plug-ins table"""

    display_if_empty = True

    @property
    def data_attributes(self):
        attributes = super().data_attributes
        sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
        attributes['table'].update({
            'data-ams-location': absolute_url(sm, self.request),
            'data-ams-order': '1,asc'
        })
        return attributes


@adapter_config(required=(ISecurityManager, IAdminLayer, SecurityPluginsTable),
                provides=IValues)
class SecurityPluginsTableValues(ContextRequestViewAdapter):
    """Security plug-ins values adapter"""

    @property
    def values(self):
        """Security plugins table values getter"""
        yield from self.context.values()


@adapter_config(name='search',
                required=(ISecurityManager, IAdminLayer, SecurityPluginsTable),
                provides=IColumn)
class SecurityPluginSearchColumn(ActionColumn):
    """Security plug-in search column"""

    hint = _("Search into folder")
    icon_class = 'fas fa-search'
    weight = 1

    href = 'search.html'
    target = '#content'
    modal_target = False

    def render_cell(self, item):
        if IDirectorySearchPlugin.providedBy(item):
            return super().render_cell(item)
        return ''


@adapter_config(name='name',
                required=(ISecurityManager, IAdminLayer, SecurityPluginsTable),
                provides=IColumn)
class SecurityPluginNameColumn(NameColumn):
    """Security plug-in name column"""


@adapter_config(name='enabled',
                required=(ISecurityManager, IAdminLayer, SecurityPluginsTable),
                provides=IColumn)
class SecurityPluginEnabledColumn(IconColumn):
    """Local users search trash column"""

    hint = _("Plug-in is enabled")
    icon_class = 'fa fa-check'

    weight = 90

    @staticmethod
    def checker(item):
        """Enabled column checker"""
        return item.enabled


@adapter_config(name='trash',
                required=(ISecurityManager, IAdminLayer, SecurityPluginsTable),
                provides=IColumn)
class SecurityPluginTrashColumn(TrashColumn):
    """Security plug-in trash column"""

    permission = MANAGE_SECURITY_PERMISSION

    def has_permission(self, item):
        if item.__name__ in (ADMIN_USER_NAME, INTERNAL_USER_NAME):
            return False
        return super().has_permission(item)


@adapter_config(required=(ISecurityManager, IAdminLayer, Interface),
                provides=ITableElementEditor)
class SecurityManagerTableElementEditor(TableElementEditor):
    """Security manager table element editor"""

    view_name = 'admin#security-plugins.html'
    modal_target = False

    def __new__(cls, context, request, view):  # pylint: disable=unused-argument
        if not request.has_permission(MANAGE_SECURITY_PERMISSION, context=context):
            return None
        return TableElementEditor.__new__(cls)


@pagelet_config(name='security-plugins.html',
                context=ISecurityManager, layer=IPyAMSLayer,
                permission=MANAGE_SECURITY_PERMISSION)
@implementer(ISecurityManagerView)
class SecurityPluginsView(TableAdminView):
    """Security plug-ins view"""

    title = _("Security plug-ins")
    table_class = SecurityPluginsTable
    table_label = _("List of security plug-ins")


@view_config(name='delete-element.json',
             context=ISecurityManager, request_type=IPyAMSLayer,
             permission=MANAGE_SECURITY_PERMISSION, renderer='json', xhr=True)
def delete_security_plugin(request):
    """Delete security plugin"""
    return delete_container_element(request)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.plugin.admin module

Internal authentication plug-ins management views.
"""

from pyams_form.ajax import ajax_form_config
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import MANAGE_SECURITY_PERMISSION
from pyams_security.interfaces.plugin import ADMIN_AUTHENTICATION_PLUGIN_LABEL, \
    IAdminAuthenticationPlugin
from pyams_security_views.zmi import SecurityPluginsTable
from pyams_security_views.zmi.plugin import SecurityPluginAddForm, SecurityPluginAddMenu, \
    SecurityPluginPropertiesEditForm
from pyams_skin.interfaces.viewlet import IHelpViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


@viewlet_config(name='add-admin-authentication-plugin.menu',
                context=ISecurityManager, layer=IAdminLayer, view=SecurityPluginsTable,
                manager=IContextAddingsViewletManager, weight=10,
                permission=MANAGE_SECURITY_PERMISSION)
class AdminAuthenticationPluginAddMenu(SecurityPluginAddMenu):
    """Admin authentication plug-in add menu"""

    label = _("Add admin authentication...")
    href = 'add-admin-authentication-plugin.html'


@ajax_form_config(name='add-admin-authentication-plugin.html',
                  context=ISecurityManager, layer=IPyAMSLayer,
                  permission=MANAGE_SECURITY_PERMISSION)
class AdminAuthenticationPluginAddForm(SecurityPluginAddForm):
    """Admin authentication plug-in add form"""

    content_factory = IAdminAuthenticationPlugin
    content_label = ADMIN_AUTHENTICATION_PLUGIN_LABEL


@ajax_form_config(name='properties.html',
                  context=IAdminAuthenticationPlugin, layer=IPyAMSLayer)
class AdminAuthenticationPluginPropertiesEditForm(SecurityPluginPropertiesEditForm):
    """Admin authentication plug-in properties editor adapter"""

    plugin_interface = IAdminAuthenticationPlugin

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if 'password' in self.widgets:
            alert = AdminAuthenticationPluginPasswordAlert(self.context, self.request, self, None)
            self.widgets['password'].suffix = alert
            self.widgets['password'].autocomplete = 'new-password'


@viewlet_config(name='properties.help', context=IAdminAuthenticationPlugin,
                layer=IAdminLayer, view=AdminAuthenticationPluginPropertiesEditForm,
                manager=IHelpViewletManager)
class AdminAuthenticationPluginPropertiesAlertMessage(AlertMessage):
    """Admin authentication plug-in properties alert message"""

    status = 'danger'
    icon_class = 'fas fa-exclamation-triangle'
    header = _("Warning!")
    _message = _("Before disabling plug-in, you should verify that you have other administration "
                 "access!")


class AdminAuthenticationPluginPasswordAlert(AlertMessage):
    """Admin authentication plug-in password alert message"""

    status = 'warning'
    css_class = 'mt-1 p-2'
    _message = _("<strong>Be careful to redefine the password when using some browsers which "
                 "are forcing autocompletion!!!</strong><br />"
                 "Clearing the password will keep the plug-in active but without the option "
                 "to log-in with this account; setting the password value to '*****' will leave "
                 "the password as-is...")
    message_renderer = 'raw'



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.plugin.group module

Internal groups management views and content providers.
"""

from pyramid.events import subscriber
from pyramid.view import view_config
from zope.interface import Interface, Invalid

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces import ISecurityManager, IViewContextPermissionChecker
from pyams_security.interfaces.base import MANAGE_SECURITY_PERMISSION
from pyams_security.interfaces.plugin import GROUPS_FOLDER_PLUGIN_LABEL, IGroupsFolderPlugin, \
    ILocalGroup
from pyams_security_views.zmi import SecurityPluginsTable
from pyams_security_views.zmi.plugin import InnerSecurityPluginFormMixin, SecurityPluginAddForm, \
    SecurityPluginAddMenu, SecurityPluginPropertiesEditForm
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_table.column import GetAttrColumn
from pyams_table.interfaces import IColumn
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.container import delete_container_element
from pyams_zmi.helper.event import get_json_table_row_add_callback, \
    get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager, IToolbarViewletManager
from pyams_zmi.table import I18nColumnMixin, Table, TableAdminView, TableElementEditor, \
    TrashColumn


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


@viewlet_config(name='add-groups-folder-plugin.menu',
                context=ISecurityManager, layer=IAdminLayer, view=SecurityPluginsTable,
                manager=IContextAddingsViewletManager, weight=30,
                permission=MANAGE_SECURITY_PERMISSION)
class GroupsFolderPluginAddMenu(SecurityPluginAddMenu):
    """Groups folder plug-in add menu"""

    label = _("Add groups folder...")
    href = 'add-groups-folder-plugin.html'


@ajax_form_config(name='add-groups-folder-plugin.html',
                  context=ISecurityManager, layer=IPyAMSLayer,
                  permission=MANAGE_SECURITY_PERMISSION)
class GroupsFolderPluginAddForm(SecurityPluginAddForm):
    """Groups folder plug-in add form"""

    content_factory = IGroupsFolderPlugin
    content_label = GROUPS_FOLDER_PLUGIN_LABEL


@ajax_form_config(name='properties.html',
                  context=IGroupsFolderPlugin, layer=IPyAMSLayer)
class GroupsFolderPropertiesEditForm(SecurityPluginPropertiesEditForm):
    """Groups folder plug-in properties edit form"""

    plugin_interface = IGroupsFolderPlugin


#
# Groups folder table view
#

class GroupsFolderGroupsTable(Table):
    """Groups folders table"""

    display_if_empty = True


@adapter_config(name='title',
                required=(IGroupsFolderPlugin, IAdminLayer, GroupsFolderGroupsTable),
                provides=IColumn)
class GroupsFolderGroupsTitleColumn(I18nColumnMixin, GetAttrColumn):
    """Groups folder groups title column"""

    i18n_header = _("Title")
    attr_name = 'title'

    weight = 10


@adapter_config(name='trash',
                required=(IGroupsFolderPlugin, IAdminLayer, GroupsFolderGroupsTable),
                provides=IColumn)
class GroupsFolderGroupsTrashColumn(TrashColumn):
    """Groups folder trash column"""

    permission = MANAGE_SECURITY_PERMISSION


@pagelet_config(name='search.html',
                context=IGroupsFolderPlugin, layer=IPyAMSLayer,
                permission=MANAGE_SECURITY_PERMISSION)
class GroupsFolderGroupsView(TableAdminView):
    """Groups folders view"""

    # pylint: disable=no-member

    @property
    def title(self):
        """View title getter"""
        return self.context.title

    @property
    def back_url(self):
        """View back URL getter"""
        manager = get_utility(ISecurityManager)
        return absolute_url(manager, self.request, 'security-plugins.html')

    table_class = GroupsFolderGroupsTable
    table_label = _("List of folder groups")


@view_config(name='delete-element.json',
             context=IGroupsFolderPlugin, request_type=IPyAMSLayer,
             permission=MANAGE_SECURITY_PERMISSION, renderer='json', xhr=True)
def delete_group(request):
    """Delete local group"""
    return delete_container_element(request)


@adapter_config(required=(ILocalGroup, IAdminLayer, Interface),
                provides=ITableElementEditor)
class GroupElementEditor(TableElementEditor):
    """Security manager table element editor"""


@adapter_config(required=ILocalGroup, provides=IViewContextPermissionChecker)
class LocalGroupPermissionChecker(ContextAdapter):
    """Local group permission checker"""

    edit_permission = MANAGE_SECURITY_PERMISSION


#
# Local groups views
#

@viewlet_config(name='add-group.action',
                context=IGroupsFolderPlugin, layer=IAdminLayer,
                view=GroupsFolderGroupsTable, manager=IToolbarViewletManager, weight=10,
                permission=MANAGE_SECURITY_PERMISSION)
class LocalGroupAddAction(ContextAddAction):
    """Local group add action"""

    label = _("Add group")
    href = 'add-group.html'


@ajax_form_config(name='add-group.html',
                  context=IGroupsFolderPlugin, layer=IPyAMSLayer,
                  permission=MANAGE_SECURITY_PERMISSION)
class LocalGroupAddForm(InnerSecurityPluginFormMixin, AdminModalAddForm):
    """Local group add form"""

    legend = _("Add local group")

    fields = Fields(ILocalGroup).omit('__parent__', '__name__')
    content_factory = ILocalGroup

    def update_content(self, obj, data):
        obj.group_id = data.get(self, {}).get('group_id')
        return super().update_content(obj, data)

    def add(self, obj):
        self.context[obj.group_id] = obj

    def next_url(self):
        return absolute_url(self.context, self.request, 'search.html')


@subscriber(IDataExtractedEvent, form_selector=LocalGroupAddForm)
def extract_local_group_add_form_data(event):
    """Check new local group form data"""
    data = event.data
    folder = event.form.context
    if data.get('group_id') in folder:
        event.form.widgets.errors += (Invalid(_("Specified login is already used.")),)


@adapter_config(required=(IGroupsFolderPlugin, IAdminLayer, LocalGroupAddForm),
                provides=IAJAXFormRenderer)
class LocalGroupAddFormRenderer(ContextRequestViewAdapter):
    """Local group add form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        return {
            'callbacks': [
                get_json_table_row_add_callback(self.context, self.request,
                                                GroupsFolderGroupsTable, changes)
            ]
        }


@ajax_form_config(name='properties.html',
                  context=ILocalGroup, layer=IPyAMSLayer)
class LocalGroupEditForm(InnerSecurityPluginFormMixin, AdminModalEditForm):
    """Local group edit form"""

    @property
    def title(self):
        """Form title getter"""
        translate = self.request.localizer.translate
        return '{}<br /><small>{}</small>'.format(
            super().title,
            translate(_("Group: {}")).format(self.context.title))

    legend = _("Group properties")

    fields = Fields(ILocalGroup).omit('__parent__', '__name__')


@adapter_config(required=(ILocalGroup, IAdminLayer, LocalGroupEditForm),
                provides=IAJAXFormRenderer)
class LocalGroupEditFormRenderer(ContextRequestViewAdapter):
    """Local group edit form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        folder = get_parent(self.context, IGroupsFolderPlugin)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(folder, self.request,
                                                    GroupsFolderGroupsTable, self.context)
            ]
        }



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.plugin.userfolder module

Local users management views.
"""

from datetime import datetime
from pyramid.events import subscriber
from pyramid.view import view_config
from zope.interface import Interface, Invalid

from pyams_form.ajax import ajax_form_config
from pyams_form.browser.checkbox import SingleCheckBoxFieldWidget
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.interfaces import DISPLAY_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces import ISecurityManager, IViewContextPermissionChecker
from pyams_security.interfaces.base import MANAGE_SECURITY_PERMISSION
from pyams_security.interfaces.plugin import ILocalUser, IUsersFolderPlugin, \
    USERS_FOLDER_PLUGIN_LABEL
from pyams_security_views.zmi import SecurityPluginsTable
from pyams_security_views.zmi.plugin import InnerSecurityPluginFormMixin, SecurityPluginAddForm, \
    SecurityPluginAddMenu, SecurityPluginPropertiesEditForm
from pyams_skin.schema.button import ActionButton, SubmitButton
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_table.column import GetAttrColumn
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.container import delete_container_element
from pyams_zmi.helper.event import get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IModalDisplayFormButtons, IModalEditFormButtons
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager, IToolbarViewletManager
from pyams_zmi.search import SearchForm, SearchResultsView, SearchView
from pyams_zmi.table import I18nColumnMixin, IconColumn, Table, TableElementEditor, TrashColumn


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


@viewlet_config(name='add-users-folder-plugin.menu',
                context=ISecurityManager, layer=IAdminLayer, view=SecurityPluginsTable,
                manager=IContextAddingsViewletManager, weight=20,
                permission=MANAGE_SECURITY_PERMISSION)
class UsersFolderPluginAddMenu(SecurityPluginAddMenu):
    """Users folder plug-in add menu"""

    label = _("Add local users folder...")
    href = 'add-users-folder-plugin.html'


@ajax_form_config(name='add-users-folder-plugin.html',
                  context=ISecurityManager, layer=IPyAMSLayer,
                  permission=MANAGE_SECURITY_PERMISSION)
class UsersFolderPluginAddForm(SecurityPluginAddForm):
    """Users folder plug-in add form"""

    content_factory = IUsersFolderPlugin
    content_label = USERS_FOLDER_PLUGIN_LABEL


@ajax_form_config(name='properties.html',
                  context=IUsersFolderPlugin, layer=IPyAMSLayer)
class UsersFolderPropertiesEditForm(SecurityPluginPropertiesEditForm):
    """Users folder plug-in properties edit form"""

    plugin_interface = IUsersFolderPlugin


#
# Local users folder search view
#

class LocalUsersSearchForm(SearchForm):  # pylint: disable=abstract-method
    """Local users search form"""

    title = _("Users search form")

    @property
    def back_url(self):
        """Form back URL getter"""
        manager = get_utility(ISecurityManager)
        return absolute_url(manager, self.request, 'security-plugins.html')


@pagelet_config(name='search.html', context=IUsersFolderPlugin, layer=IPyAMSLayer,
                permission=MANAGE_SECURITY_PERMISSION)
class LocalUsersSearchView(SearchView):
    """Local users search view"""

    title = _("Users search form")
    search_form = LocalUsersSearchForm


class LocalUsersSearchResultsTable(Table):
    """Local users search results table"""


@adapter_config(required=(IUsersFolderPlugin, IAdminLayer, LocalUsersSearchResultsTable),
                provides=IValues)
class LocalUsersSearchResultsValues(ContextRequestViewAdapter):
    """Local users search results values"""

    @property
    def values(self):
        """Local users search table results getter"""
        yield from self.context.get_search_results({
            'query': self.request.params.get('form.widgets.query')
        })


@adapter_config(name='login',
                required=(IUsersFolderPlugin, IAdminLayer, LocalUsersSearchResultsTable),
                provides=IColumn)
class LocalUsersSearchLoginColumn(I18nColumnMixin, GetAttrColumn):
    """Local users search login column"""

    i18n_header = _("Login")
    attr_name = 'login'

    weight = 10


@adapter_config(name='title',
                required=(IUsersFolderPlugin, IAdminLayer, LocalUsersSearchResultsTable),
                provides=IColumn)
class LocalUsersSearchTitleColumn(I18nColumnMixin, GetAttrColumn):
    """Local users search title column"""

    i18n_header = _("Title")
    attr_name = 'title'

    weight = 20


@adapter_config(name='email',
                required=(IUsersFolderPlugin, IAdminLayer, LocalUsersSearchResultsTable),
                provides=IColumn)
class LocalUsersSearchEmailColumn(I18nColumnMixin, GetAttrColumn):
    """Local users search email column"""

    i18n_header = _("Mail address")
    attr_name = 'email'

    weight = 30


@adapter_config(name='password',
                required=(IUsersFolderPlugin, IAdminLayer, LocalUsersSearchResultsTable),
                provides=IColumn)
class LocalUsersSearchPasswordColumn(IconColumn):
    """Local users search password column"""

    hint = _("Account has password set")
    icon_class = 'fas fa-key'

    weight = 180

    @staticmethod
    def checker(item):
        """Password column checker"""
        return item.password


@adapter_config(name='active',
                required=(IUsersFolderPlugin, IAdminLayer, LocalUsersSearchResultsTable),
                provides=IColumn)
class LocalUsersSearchActiveColumn(IconColumn):
    """Local users search active column"""

    hint = _("Account is activated")
    icon_class = 'fa fa-check'

    weight = 200

    @staticmethod
    def checker(item):
        """Active column checker"""
        return item.activated


@adapter_config(name='trash',
                required=(IUsersFolderPlugin, IAdminLayer, LocalUsersSearchResultsTable),
                provides=IColumn)
class LocalUsersSearchTrashColumn(TrashColumn):
    """Local users search trash column"""


@pagelet_config(name='search-results.html', context=IUsersFolderPlugin, layer=IPyAMSLayer,
                permission=MANAGE_SECURITY_PERMISSION, xhr=True)
class LocalUsersSearchResultsView(SearchResultsView):
    """Local users search results view"""

    table_label = _("Search results")
    table_class = LocalUsersSearchResultsTable


@view_config(name='delete-element.json', context=IUsersFolderPlugin, request_type=IPyAMSLayer,
             permission=MANAGE_SECURITY_PERMISSION, renderer='json', xhr=True)
def delete_local_user(request):
    """Local user delete view"""
    return delete_container_element(request)


#
# Local user views
#

@viewlet_config(name='add-user.action', context=IUsersFolderPlugin, layer=IAdminLayer,
                view=LocalUsersSearchForm, manager=IToolbarViewletManager, weight=10,
                permission=MANAGE_SECURITY_PERMISSION)
class LocalUserAddAction(ContextAddAction):
    """Local user add action"""

    label = _("Add user")
    href = 'add-user.html'


@ajax_form_config(name='add-user.html', context=IUsersFolderPlugin, layer=IPyAMSLayer,
                  permission=MANAGE_SECURITY_PERMISSION)
class LocalUserAddForm(InnerSecurityPluginFormMixin, AdminModalAddForm):
    """Local user add form"""

    legend = _("Add local user")

    fields = Fields(ILocalUser).select('login', 'email', 'firstname', 'lastname',
                                       'company_name', 'password_manager', 'password',
                                       'confirmed_password', 'wait_confirmation')
    fields['wait_confirmation'].widget_factory = SingleCheckBoxFieldWidget

    content_factory = ILocalUser

    def update_content(self, obj, data):
        data = data.get(self, {})
        for attr in ('login', 'email', 'firstname', 'lastname', 'company_name',
                     'password_manager', 'password', 'wait_confirmation'):
            setattr(obj, attr, data.get(attr))
        obj.self_registered = False
        if obj.wait_confirmation:
            obj.generate_secret()
        else:
            obj.activation_date = tztime(datetime.utcnow())
            obj.activated = True

    def add(self, obj):
        self.context[obj.login] = obj


@subscriber(IDataExtractedEvent, form_selector=LocalUserAddForm)
def extract_local_user_add_form_data(event):
    """Check new local user form data"""
    data = event.data
    folder = event.form.context
    if not folder.check_login(data.get('login')):
        event.form.widgets.errors += (Invalid(_("Specified login is already used.")),)
    password = data.get('password')
    if password and (password != data.get('confirmed_password')):
        event.form.widgets.errors += (Invalid(_("User password was not confirmed correctly.")),)


@adapter_config(required=(IUsersFolderPlugin, IAdminLayer, LocalUserAddForm),
                provides=IAJAXFormRenderer)
class LocalUserAddFormRenderer(ContextRequestViewAdapter):
    """Local user add form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        return {
            'status': 'success',
            'message': self.request.localizer.translate(_("New user was created "
                                                          "successfully!"))
        }


class ILocalUserEditFormButtons(IModalEditFormButtons):
    """Local user edit form buttons"""

    change_password = ActionButton(name='change_password',
                                   title=_("Change password"))

    refresh = SubmitButton(name='refresh',
                           title=_("Refresh secret"))

    enable = SubmitButton(name='enable',
                          title=_("Enable"),
                          condition=lambda form: not form.context.activated)

    disable = SubmitButton(name='disable',
                           title=_("Disable"),
                           condition=lambda form: form.context.activated)


@ajax_form_config(name='properties.html', context=ILocalUser, layer=IPyAMSLayer)
class LocalUserEditForm(InnerSecurityPluginFormMixin, AdminModalEditForm):
    """Local user edit form"""

    @property
    def title(self):
        """Form title getter"""
        translate = self.request.localizer.translate
        return '{}<br /><small>{}</small>'.format(
            super().title,
            translate(_("User: {}")).format(self.context.title))

    legend = _("User properties")

    fields = Fields(ILocalUser).omit('__parent__', '__name__', 'password', 'confirmed_password',
                                     'password_manager')
    fields['wait_confirmation'].widget_factory = SingleCheckBoxFieldWidget
    fields['self_registered'].widget_factory = SingleCheckBoxFieldWidget
    fields['activated'].widget_factory = SingleCheckBoxFieldWidget

    @property
    def buttons(self):
        if self.mode == DISPLAY_MODE:
            return Buttons(IModalDisplayFormButtons)
        return Buttons(ILocalUserEditFormButtons).select('change_password', 'apply',
                                                         'refresh', 'enable', 'disable',
                                                         'close')

    ajax_form_target = None

    def update_actions(self):
        super().update_actions()
        translate = self.request.localizer.translate
        if 'change_password' in self.actions:
            action = self.actions['change_password']
            action.add_class('btn-info mr-auto')
            action.href = absolute_url(self.context, self.request, 'change-password.html')
            action.modal_target = True
        if 'refresh' in self.actions:
            action = self.actions['refresh']
            action.add_class('btn-secondary')
            action.hint = translate(_("Refresh activation secret and send a new activation link "
                                      "to this user"))
        if 'enable' in self.actions:
            action = self.actions['enable']
            action.add_class('btn-secondary')
            action.hint = translate(_("Re-enable user profile"))
        if 'disable' in self.actions:
            action = self.actions['disable']
            action.add_class('btn-danger')
            action.hint = translate(_("Disable user profile temporarily will forbid login with "
                                      "this account..."))

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if ('wait_confirmation' in self.widgets) and self.context.activated:
            self.widgets['wait_confirmation'].mode = DISPLAY_MODE
        if 'activation_secret' in self.widgets:
            self.widgets['activation_secret'].mode = DISPLAY_MODE
        if 'activation_hash' in self.widgets:
            self.widgets['activation_hash'].mode = DISPLAY_MODE
        if 'activated' in self.widgets:
            self.widgets['activated'].mode = DISPLAY_MODE
        if 'activation_date' in self.widgets:
            self.widgets['activation_date'].mode = DISPLAY_MODE

    @handler(ILocalUserEditFormButtons['apply'])
    def handle_apply(self, action):
        super().handle_apply(self, action)  # pylint: disable=too-many-function-args

    @handler(ILocalUserEditFormButtons['refresh'])
    def handle_refresh(self, action):
        """Refresh button handler"""
        user = self.context
        user.refresh_secret(notify=True, request=self.request)
        self.finished_state.update({
            'action': action,
            'changes': user
        })

    @handler(ILocalUserEditFormButtons['enable'])
    def handle_enable(self, action):
        """Enable button handler"""
        user = self.get_content()
        user.wait_confirmation = False
        user.activated = True
        self.finished_state.update({
            'action': action,
            'changes': user
        })

    @handler(ILocalUserEditFormButtons['disable'])
    def handle_disable(self, action):
        """Disable button handler"""
        user = self.get_content()
        user.activated = False
        self.finished_state.update({
            'action': action,
            'changes': user
        })


@adapter_config(name='apply',
                required=(ILocalUser, IAdminLayer, LocalUserEditForm),
                provides=IAJAXFormRenderer)
class LocalUserEditFormApplyActionRenderer(ContextRequestViewAdapter):
    """Local user edit form 'apply' action renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        folder = get_parent(self.context, IUsersFolderPlugin)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(folder, self.request,
                                                    LocalUsersSearchResultsTable, self.context)
            ]
        }


@adapter_config(name='refresh',
                required=(ILocalUser, IAdminLayer, LocalUserEditForm),
                provides=IAJAXFormRenderer)
class LocalUserEditFormRefreshActionRenderer(ContextRequestViewAdapter):
    """Local user edit form 'refresh' action renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        folder = get_parent(self.context, IUsersFolderPlugin)
        return {
            'status': 'success',
            'message': self.request.localizer.translate(_("This profile has been deactivated!"
                                                          "<br /> "
                                                          "A new activation code has been "
                                                          "created and sent to this user!")),
            'callbacks': [
                get_json_table_row_refresh_callback(folder, self.request,
                                                    LocalUsersSearchResultsTable, self.context)
            ]
        }


@adapter_config(name='enable',
                required=(ILocalUser, IAdminLayer, LocalUserEditForm),
                provides=IAJAXFormRenderer)
class LocalUserEditFormEnableActionRenderer(ContextRequestViewAdapter):
    """Local user edit form 'enable' action renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        translate = self.request.localizer.translate
        folder = get_parent(self.context, IUsersFolderPlugin)
        return {
            'status': 'success',
            'messagebox': {
                'status': 'success',
                'timeout': 10000,
                'title': translate(_("Enabled user!")),
                'icon': 'fas fa-info-circle',
                'message': translate(_("User has been enabled and can now log-in!"))
            },
            'callbacks': [
                get_json_table_row_refresh_callback(folder, self.request,
                                                    LocalUsersSearchResultsTable, self.context)
            ]
        }


@adapter_config(name='disable',
                required=(ILocalUser, IAdminLayer, LocalUserEditForm),
                provides=IAJAXFormRenderer)
class LocalUserEditFormDisableActionRenderer(ContextRequestViewAdapter):
    """Local user edit form 'disable' action renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        translate = self.request.localizer.translate
        folder = get_parent(self.context, IUsersFolderPlugin)
        return {
            'status': 'success',
            'messagebox': {
                'status': 'warning',
                'timeout': 10000,
                'title': translate(_("Disabled user!")),
                'icon': 'fas fa-info-circle',
                'message': translate(_("User has been disabled and can't log in anymore!"))
            },
            'callbacks': [
                get_json_table_row_refresh_callback(folder, self.request,
                                                    LocalUsersSearchResultsTable, self.context)
            ]
        }


@adapter_config(required=(ILocalUser, IAdminLayer, Interface),
                provides=ITableElementEditor)
class LocalUserEditor(TableElementEditor):
    """Local user editor adapter"""


@adapter_config(required=ILocalUser, provides=IViewContextPermissionChecker)
class LocalUserPermissionChecker(ContextAdapter):
    """Local user permission checker"""

    edit_permission = MANAGE_SECURITY_PERMISSION


@ajax_form_config(name='change-password.html', context=ILocalUser, layer=IPyAMSLayer,
                  permission=MANAGE_SECURITY_PERMISSION)
class LocalUserPasswordChangeForm(InnerSecurityPluginFormMixin, AdminModalEditForm):
    """Local user password change form"""

    @property
    def title(self):
        """Form title getter"""
        translate = self.request.localizer.translate
        return '{}<br /><small>{}</small>'.format(
            super().title,
            translate(_("User: {}")).format(self.context.title))

    legend = _("Change user password")

    fields = Fields(ILocalUser).select('password_manager', 'password', 'confirmed_password')


@subscriber(IDataExtractedEvent, form_selector=LocalUserPasswordChangeForm)
def extract_local_user_password_form_data(event):
    """Check new local user password data"""
    data = event.data
    password = data.get('password')
    if password and (password != data.get('confirmed_password')):
        event.form.widgets.errors += (Invalid(_("User password was not confirmed correctly.")),)
    else:
        del data['confirmed_password']



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.plugin base module

This module defines classes which are used by all security plug-ins management views.
"""

from pyramid.events import subscriber
from zope.interface import Interface, Invalid

from pyams_form.browser.checkbox import SingleCheckBoxFieldWidget
from pyams_form.field import Fields
from pyams_form.interfaces import DISPLAY_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_security.interfaces import IPlugin, ISecurityManager, IViewContextPermissionChecker
from pyams_security.interfaces.base import MANAGE_SECURITY_PERMISSION
from pyams_security_views.zmi import SecurityPluginsTable
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.event import get_json_table_row_add_callback, \
    get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_label


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


class SecurityPluginAddMenu(MenuItem):
    """Security manager plug-in add form"""

    modal_target = True

    def get_href(self):
        sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
        return absolute_url(sm, self.request, self.href)


class SecurityPluginAddForm(AdminModalAddForm):
    """Security plug-in add form"""

    @property
    def title(self):
        """Add form title getter"""
        translate = self.request.localizer.translate
        return '<small>{}</small><br />{}'.format(
            get_object_label(self.context, self.request, self),
            translate(_("New plug-in: {}")).format(translate(self.content_label)))

    legend = _("New plug-in properties")
    content_factory = IPlugin
    content_label = '--'

    object_data = {
        'ams-warn-on-change': False
    }

    @property
    def fields(self):
        """Form fields getter"""
        fields = Fields(self.content_factory).omit('__parent__', '__name__')
        fields['enabled'].widget_factory = SingleCheckBoxFieldWidget
        return fields

    def add(self, obj):
        sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
        sm[obj.prefix] = obj

    def next_url(self):
        return absolute_url(self.context, self.request, 'admin#security-plugins.html')


@subscriber(IDataExtractedEvent, form_selector=SecurityPluginAddForm)
def extract_plugin_add_form_data(event):
    """Security plug-in add form data extraction"""
    data = event.data
    sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
    if data.get('prefix') in sm:
        event.form.widgets.errors += (Invalid(_("Specified prefix is already used!")),)


@adapter_config(required=(ISecurityManager, IAdminLayer, SecurityPluginAddForm),
                provides=IAJAXFormRenderer)
class SecurityPluginAddFormRenderer(ContextRequestViewAdapter):
    """Security plug-in add form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:  # WARNING: creating an empty container will return a "false" value!
            return None
        return {
            'callbacks': [
                get_json_table_row_add_callback(self.request.root, self.request,
                                                SecurityPluginsTable, changes)
            ]
        }


@adapter_config(required=IPlugin,
                provides=IObjectLabel)
def security_plugin_label(context):
    """Security plug-in name adapter"""
    return context.title


class InnerSecurityPluginFormMixin:
    """Inner security plug-in form mixin"""

    @property
    def title(self):
        """Form title getter"""
        translate = self.request.localizer.translate
        manager = get_utility(ISecurityManager)
        plugin = get_parent(self.context, IPlugin)
        return '<small>{}</small><br />{}'.format(
            get_object_label(manager, self.request, self),
            translate(_("Plug-in: {}")).format(get_object_label(plugin, self.request, self)))


@adapter_config(required=(IPlugin, IAdminLayer, Interface),
                provides=ITableElementEditor)
class SecurityPluginEditor(TableElementEditor):
    """Security plug-in editor adapter"""


@adapter_config(required=IPlugin, provides=IViewContextPermissionChecker)
class SecurityManagerPluginPermissionChecker(ContextAdapter):
    """Security manager plug-in permission checker"""

    edit_permission = MANAGE_SECURITY_PERMISSION


class SecurityPluginPropertiesEditForm(AdminModalEditForm):
    """Security plug-in properties editor adapter"""

    @property
    def title(self):
        """Plug-in edit form title getter"""
        translate = self.request.localizer.translate
        manager = get_utility(ISecurityManager)
        plugin = get_parent(self.context, IPlugin)
        return '<small>{}</small><br />{}'.format(
            get_object_label(manager, self.request, self),
            translate(_("Plug-in: {}")).format(get_object_label(plugin, self.request, self)))

    legend = _("Plug-in properties")

    plugin_interface = IPlugin
    content_label = '--'

    @property
    def fields(self):
        """Form fields getter"""
        fields = Fields(self.plugin_interface).omit('__parent__', '__name__')
        fields['enabled'].widget_factory = SingleCheckBoxFieldWidget
        return fields

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if 'prefix' in self.widgets:
            self.widgets['prefix'].mode = DISPLAY_MODE


@adapter_config(required=(IPlugin, IAdminLayer, SecurityPluginPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SecurityPluginPropertiesAJAXRenderer(ContextRequestViewAdapter):
    """Security plugin properties AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        return get_json_table_row_refresh_callback(self.request.root, self.request,
                                                   SecurityPluginsTable, self.context)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.site module

This module provides a default security context from site root.
"""

from zope.interface import Interface

from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config


__docformat__ = 'restructuredtext'


@adapter_config(required=(ISiteRoot, IPyAMSLayer, Interface),
                provides=IViewContextPermissionChecker)
class SiteRootPermissionChecker(ContextRequestViewAdapter):
    """Site root permission checker"""

    edit_permission = MANAGE_SYSTEM_PERMISSION



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.manager module

This module provides views and content providers used to manage security manager properties.
"""

from zope.interface import Interface, implementer

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IGroup, IInnerSubForm
from pyams_form.subform import InnerEditForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import MANAGE_SECURITY_PERMISSION
from pyams_security_views.zmi import ISecurityMenu
from pyams_security_views.zmi.interfaces import ISecurityPropertiesEditForm
from pyams_security_views.zmi.widget import SecurityManagerPluginsFieldWidget
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


@adapter_config(required=(ISecurityManager, IAdminLayer, Interface),
                provides=IObjectLabel)
def security_manager_label(context, request, view):
    """Security manager label getter"""
    return request.localizer.translate(_("Security manager"))


@viewlet_config(name='security-properties.menu',
                context=ISecurityManager, layer=IAdminLayer,
                manager=ISecurityMenu, weight=10,
                permission=MANAGE_SECURITY_PERMISSION)
class SecurityPropertiesMenu(NavigationMenuItem):
    """Security manager properties menu"""

    label = _("Properties")
    href = '#security-properties.html'


@ajax_form_config(name='security-properties.html',
                  context=ISecurityManager, layer=IPyAMSLayer,
                  permission=MANAGE_SECURITY_PERMISSION)
@implementer(ISecurityPropertiesEditForm)
class SecurityPropertiesEditForm(AdminEditForm):
    """Security manager properties edit form"""

    title = _("Security manager")
    legend = _("Properties")

    fields = Fields(ISecurityManager).select('credentials_plugins_names',
                                             'authentication_plugins_names',
                                             'directory_plugins_names')
    fields['credentials_plugins_names'].widget_factory = SecurityManagerPluginsFieldWidget
    fields['authentication_plugins_names'].widget_factory = SecurityManagerPluginsFieldWidget
    fields['directory_plugins_names'].widget_factory = SecurityManagerPluginsFieldWidget


@adapter_config(name='security-registration',
                required=(ISecurityManager, IAdminLayer, SecurityPropertiesEditForm),
                provides=IGroup)
class SecurityRegistrationGroup(FormGroupChecker):
    """Security manager registration fields"""

    fields = Fields(ISecurityManager).select('open_registration', 'users_folder')


@viewlet_config(name='security-registration.header',
                context=ISecurityManager, layer=IAdminLayer, view=SecurityRegistrationGroup,
                manager=IHeaderViewletManager, weight=1)
class SecurityRegistrationHeader(AlertMessage):
    """Security registration header"""

    status = 'info'

    _message = _("Open registration can be used when you want external users to be able to "
                 "freely register their user account.\n"
                 "You then have to select the users folder into which their profile will be "
                 "stored.\n"
                 "THIS CAN BE DANGEROUS! You should enable this feature carefully...")


@adapter_config(name='security-zmi',
                required=(ISecurityManager, IAdminLayer, SecurityPropertiesEditForm),
                provides=IInnerSubForm)
class SecurityZMIEditForm(InnerEditForm):
    """Security manager administration interface edit form"""

    legend = _("Management interface")

    fields = Fields(ISecurityManager).select('show_home_menu')



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.rest module

This module defines components which are used to handle REST security configuration
to set allowed CORS origins.
"""

from zope.interface import Interface

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import MANAGE_SECURITY_PERMISSION
from pyams_security.interfaces.rest import ICORSSecurityInfo
from pyams_security_views.zmi import ISecurityMenu
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_utility
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_security_views import _


@viewlet_config(name='cors-configuration.menu',
                context=ISecurityManager, layer=IAdminLayer,
                manager=ISecurityMenu, weight=20,
                permission=MANAGE_SECURITY_PERMISSION)
class CORSConfigurationMenu(NavigationMenuItem):
    """CORS configuration menu"""

    label = _("CORS configuration")
    href = '#cors-configuration.html'


@ajax_form_config(name='cors-configuration.html',
                  context=ISecurityManager, layer=IPyAMSLayer,
                  permission=MANAGE_SECURITY_PERMISSION)
class CORSConfigurationEditForm(AdminEditForm):
    """CORS configuration edit form"""

    title = _("CORS configuration")

    fields = Fields(Interface)

    def get_content(self):
        """Content getter"""
        return get_utility(ISecurityManager)


@adapter_config(name='cors-configuration',
                required=(ISecurityManager, IAdminLayer, CORSConfigurationEditForm),
                provides=IGroup)
class CORSConfigurationGroup(FormGroupChecker):
    """CORS configuration group"""

    fields = Fields(ICORSSecurityInfo)

    label_css_class = 'col-sm-2 col-md-3'
    input_css_class = 'col-sm-10 col-md-9'

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        origins = self.widgets.get('allowed_origins')
        if origins is not None:
            origins.rows = 10


@viewlet_config(name='cors-configuration.header',
                context=ISecurityManager, layer=IAdminLayer, view=CORSConfigurationGroup,
                manager=IHeaderViewletManager, weight=1)
class CORSConfigurationHeader(AlertMessage):
    """CORS configuration header"""

    status = 'info'

    _message = _("If you disable CORS origins check, any CORS request will be accepted.\n"
                 "Otherwise, only origins specified below will be allowed.\n"
                 "Please note that you don't have to set main site URL(s) below, only "
                 "\"external\" URLs have to be defined.")



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.interfaces module

This module defines public views and viewlet managers interfaces.
"""

from zope.interface import Interface

from pyams_form.interfaces.form import IEditForm
from pyams_skin.interfaces.widget import IOrderedListWidget
from pyams_zmi.interfaces.viewlet import INavigationMenuItem


__docformat__ = 'restructuredtext'


class ISecurityMenu(INavigationMenuItem):
    """Security menu interface"""


class IObjectSecurityMenu(INavigationMenuItem):
    """Object security menu"""


class ISecurityManagerView(Interface):
    """Security manager view"""


class ISecurityPropertiesEditForm(IEditForm):
    """Security properties edit form marker interface"""


class ISecurityManagerPluginsWidget(IOrderedListWidget):
    """Security plugins ordering widget"""



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_security_views.zmi.notifications module

This module provides notifications management views.
"""

from zope.interface import Interface

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import MANAGE_SECURITY_PERMISSION
from pyams_security.interfaces.notification import INotificationSettings
from pyams_security_views.zmi import ISecurityMenu
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_utility
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_security_views import _  # pylint: disable=ungrouped-imports


@viewlet_config(name='security-notifications.menu',
                context=ISecurityManager, layer=IAdminLayer,
                manager=ISecurityMenu, weight=15,
                permission=MANAGE_SECURITY_PERMISSION)
class SecurityNotificationsMenu(NavigationMenuItem):
    """Security manager notifications menu"""

    label = _("Notifications")
    href = '#security-notifications.html'


@ajax_form_config(name='security-notifications.html',
                  context=ISecurityManager, layer=IPyAMSLayer,
                  permission=MANAGE_SECURITY_PERMISSION)
class SecurityNotificationsEditForm(AdminEditForm):
    """Security notifications edit form"""

    title = _("Notifications settings")

    fields = Fields(Interface)

    def get_content(self):
        return get_utility(ISecurityManager)


@adapter_config(name='security-notifications',
                required=(ISecurityManager, IAdminLayer, SecurityNotificationsEditForm),
                provides=IGroup)
class SecurityNotificationsGroup(FormGroupChecker):
    """Security notifications settings group"""

    fields = Fields(INotificationSettings)


@viewlet_config(name='security-notifications.header',
                context=ISecurityManager, layer=IAdminLayer, view=SecurityNotificationsGroup,
                manager=IHeaderViewletManager, weight=1)
class SecurityNotificationsHeader(AlertMessage):
    """Security notifications header"""

    status = 'info'

    _message = _("Notifications are used to send messages to users during their registration "
                 "process.\n"
                 "Applications can also send messages during their normal process.")
